/*
Copyright (C) <2008>  Guido Steiner and Detlef Wolf
contact: fam@wolfol.de
This is free software -- GNU LGPL license applies.
see http://www.gnu.org/licenses/lgpl.txt for details
*/

import javax.swing.SwingUtilities ;

public class PendulumMover extends Thread {

  PendulumModel m ;
  PendulumPanel p ;
  PendulumControls c ;

  public PendulumMover(PendulumModel m, PendulumPanel p, PendulumControls c) {
    this.m = m ;
    this.p = p ;
    this.c = c ;
  }  

  private double getTime() {
    return (double)System.currentTimeMillis() / 1000.0 ;
  }


    /* sleeping for only mircoseconds does not seem to work on my PC:
      try { 
      sleep(0,1000) ;
      }
      catch (InterruptedException e) {
      ;
      }
      Therefore: adapt granularity of simulation steps to speed of computer 
    */


  public void run() {
    int cnt = 0 ;
    final int silentComputeSteps = 100 ;
    int i ;
    double realStartTime = getTime() ;
    double modelTime = 0.0 ;
    double realTime ;
    double ratio ;
    double initialStepTime=m.getStepTime() ;

    Runnable doRepaint = new Runnable() {
        public void run() {
          p.repaint() ;
        }
      };

    while (true) {
      ++cnt ;

      i = silentComputeSteps ;
      while (i-- != 0) {
        m.step() ;
        modelTime += m.getStepTime() ;
      }
      SwingUtilities.invokeLater(doRepaint) ;

      if (cnt % 1000 == 0) { // adapt to actual compute speed
        realTime = getTime() - realStartTime ;
        ratio = realTime / modelTime ;
        // high ratio --> computer is slow;       low ratio --> computer is fast
        //                make larger time steps                make smaller time steps
        m.setStepTime(initialStepTime * ratio / 10.0) ;
        // System.out.println("ratio r/m=" + ratio + " ts=" + stepTime + " model=" + modelTime + " real=" + realTime) ;
      }

    }   
  }
}

